<?php
	/********************************************************************/
	/*                                                                  */
	/*      Copyright (C) 2004 Arash Dejkam, All Rights Reserved.       */
	/*      http://www.tracewatch.com                                   */
	/*                                                                  */
	/*      Please read the licence file distributed with this          */
	/*      file or the one available at tracewatch.com for             */
	/*      the terms under which you can use or modify this file.      */
	/*                                                                  */
	/********************************************************************/
	
	require_once $twatch->path( 'lib/PassivePage.php' );
	require_once $twatch->path( 'data/DataGlobal.php' );
	
	
	class TwatchPage extends TwatchPassivePage {
		private $conf_data;
		private $perm = true;
		
		public $adminPage = false;
		
		public $websiteId;
		
		public $selectedLocaleId;
		
		public $beforeOutput = null;
		
		function __construct( $before_header = false ) {
			global $twatch;
			parent::__construct();
			$this->conf_data = array( TwatchConfig::$defaultProperties );
			$this->selected_top_button = 0;
			$this->before_header = false;
		}
		
		function addExtraConfig( &$data ) {
			$this->conf_data[] = &$data;
		}
		
		public function setLeftButtons() {}
		
		public function init() {
			global $twatch;
			$this->includeBaseJs();
			$this->inHead->pl( '<script type="text/javascript" src="'.$twatch->url( $this->toRoot, 'js/Global.js' ).'"></script>' );	
		}
		
		protected function includeBaseJs() {
			global $twatch;
			$this->inHead->pl( '<script type="text/javascript" src="'.$twatch->baseUrl( $this->toRoot, 'js/ArdeClass.js' ).'"></script>' );
			$this->inHead->pl( '<script type="text/javascript" src="'.$twatch->baseUrl( $this->toRoot, 'js/ArdeRequest.js' ).'"></script>' );
			$this->inHead->pl( '<script type="text/javascript" src="'.$twatch->baseUrl( $this->toRoot, 'js/ArdeComponent.js' ).'"></script>' );
		}
		
		protected function justBeforeHeader() {}
		
		public function open( ArdeXmlPrinter $p, $websiteSelect = false ) {
			global $twatch, $ardeUser, $ardeBase, $eReporter;

			$twatch->db = new ArdeDb( $twatch->settings );
			
			$twatch->db->connect();
			
			
			TwatchApp::initUser();

			if( $ardeUser->user->hasPermission( TwatchUserData::VIEW_ERRORS ) ) {
				$p->setMutedErrors(  $twatch->settings[ 'authorized_muted_errors' ] );
				$p->setHideErrors( !$twatch->settings[ 'authorized_show_errors' ] );
			}
			
			if( ( $twatch->settings[ 'authorized_log_errors' ] && $ardeUser->user->hasPermission( TwatchUserData::VIEW_ERRORS ) ) ||
				( $twatch->settings[ 'unauthorized_log_errors' ] && !$ardeUser->user->hasPermission( TwatchUserData::VIEW_ERRORS ) ) ) {
				ArdeException::setGlobalReporter( new TwatchErrorLogger( ArdeException::getGlobalReporter() ) );
			}

			
			$twatch->config = new TwatchConfig( $twatch->db );
			foreach( $this->conf_data as $k => $v ) {
				$twatch->config->addDefaults( $this->conf_data[$k] );
			}
			$twatch->config->applyAllChanges();
			
			if( isset( $_GET[ 'lang' ] ) && $twatch->localeExists( $_GET[ 'lang' ] ) ) {
				$this->selectedLocaleId = $_GET[ 'lang' ];
			} else {
				$this->selectedLocaleId = $twatch->config->get( TwatchConfig::DEFAULT_LANG );
			}
			$this->loadLocale();
			
			$p->xmlLangCode = $twatch->locale->xmlLangCode;
			
			if( isset( $_GET[ 'website' ] ) ) {
				$this->websiteId = (int)$_GET[ 'website' ];
				
				if( !$twatch->config->propertyExists( TwatchConfig::WEBSITES, $this->websiteId ) ) $this->websiteId = 1;
			} else {
				$this->websiteId = 1;
			}
			$website = $twatch->config->get( TwatchConfig::WEBSITES, $this->websiteId );
			if( $website->parent ) {
				$this->websiteId = 1;
				$website = $twatch->config->get( TwatchConfig::WEBSITES, $this->websiteId );
			}
			$this->setLeftButtons();
			
			$twatch->state = new TwatchState( $twatch->db );
			$twatch->state->addDefaults( TwatchState::$defaultProperties );
			$twatch->state->addDefaults( TwatchState::$extraDefaults );
			$twatch->state->applyAllChanges();
			
			
			$twatch->callFunction( 'twatchSetAppTime' );
			
			
			
			if( !$ardeUser->user->hasPermission( TwatchUserData::VIEW_REPORTS ) || ( $this->adminPage && !$ardeUser->user->hasPermission( TwatchUserData::VIEW_ADMIN ) ) ) {
				$url = new ArdeUrlWriter( 'login.php' );
				$url->setParam( 'back', ardeRequestUri() )->setParam( 'profile', $twatch->settings[ 'user_profile' ], 'default' );
				ardeRedirect( $twatch->extUrl( $this->toRoot, 'user', $url->getUrl() ) );
			}
			
			
			$this->title = $twatch->locale->text( $website->name ).' - '.$twatch->locale->text( $this->title );
			$this->subTitle = $twatch->locale->text( $this->subTitle );
			
			$this->justBeforeHeader();
			
			$p->start( true );
	
			$p->crossPrinter->forceHold();
			
			$this->printHeader( $p );
		
			$p->printErrorStyleAndScript = false;
			
			$p->crossPrinter->forceFlush();
			
			$this->printFooter( $p->footer );
			
			if( !$ardeUser->user->hasPermission( TwatchUserData::VIEW_REPORTS ) ) {
				throw new TwatchRestriction( 'no permission' );
			}
			
			while ( ob_get_level() > 0 ) {
				try { ob_end_flush(); } catch( Exception $e ) {}
			}
			try{ ob_implicit_flush(); } catch( Exception $e ) {}
		}
		
		public function close( ArdeXmlPrinter $p ) {
			parent::close( $p );
		}
		
		protected function loadLocale() {
			global $twatch;

				
			$twatch->loadLocale( $this->selectedLocaleId );

		}
		
		protected function printInLangCont( ArdePrinter $p ) {
			global $twatch;
			$p->pl( 'Language: <select id="lang_select">' );
			$url = ArdeUrlWriter::getCurrentRelative();
			$defaultId = $twatch->config->get( TwatchConfig::DEFAULT_LANG );
			foreach( $twatch->getLocaleIds() as $id ) {
				$url->setParam( 'lang', $id, $defaultId );
				$p->pl( '<option value="'.$url->getUrl().'"'.($id == $this->selectedLocaleId?' selected="selected"':'').'>'.$id.'</option>' );
			}
			$p->pl( '</select>' );
			$p->pl( '<script type="text/javascript">activateLinkSelect( "lang_select" );</script>' );
		}
		
		public function initJsPage( ArdePrinter $p, $texts = array() ) {
			global $twatch;
			$p->pl( 'websiteId = '.$this->websiteId.';' );
			$p->pl( "baseUrl = '".$twatch->baseUrl( $this->toRoot, '/' )."';" );
			$p->pl( "twatchUrl = '".$twatch->url( $this->toRoot, '/' )."';" );
			$p->pl( "twatchProfile = '".ArdeJs::escape($twatch->profile)."';" );
			$p->pl( 'ardeLocale = '.$twatch->locale->jsObject( $texts, $twatch->config->get( TwatchConfig::DEFAULT_LANG ) ).";" );
		}
		
		
		public function printHeader( ArdePrinter $p, $rightToLeft = false ) {
			global $twatch, $ardeUser, $twatchProfile;

			
			if( $ardeUser->user->hasPermission( TwatchUserData::VIEW_REPORTS ) ) {
				
				$profiles = $twatch->getProfiles();
				if( count( $profiles ) > 1 ) {
					$this->inList->pl( $twatch->locale->text( 'Profile' ).': <select id="profile_select">', 1 );
					$url = ArdeUrlWriter::getCurrentRelative();
					foreach( $profiles as $id => $name ) {
						$url->setParam( 'profile', $id, 'default' );
						$this->inList->pl( '<option value="'.$url->getUrl().'" '.( $id == $twatch->profile ? 'selected="selected"' : '' ).'>'.$twatch->locale->text( $name ).'</option>' );
					}
					$this->inList->rel();
					$this->inList->pl( '</select> ' );
					$this->inList->pl( '<script type="text/javascript">activateLinkSelect( "profile_select" );</script>' );
				}
				$this->inList->pl( $twatch->locale->text( 'Website' ).': <select id="website_select">', 1 );
				$url = ArdeUrlWriter::getCurrentRelative();
				foreach( $twatch->config->getList( TwatchConfig::WEBSITES ) as $website ) {
					if( $website->parent ) continue;
					$url->setParam( 'website', $website->getId(), 1 );
					$this->inList->pl( '<option value="'.$url->getUrl().'" '.( $website->getId() == $this->websiteId ? 'selected="selected"' : '' ).'>'.$twatch->locale->text( $website->name ).'</option>' );
				}
				$this->inList->rel();
				$this->inList->pl( '</select>' );
				$this->inList->pl( '<script type="text/javascript">activateLinkSelect( "website_select" );</script>' );
				
				
				$url = new ArdeUrlWriter();
				$url->setParam( 'lang', $this->selectedLocaleId, $twatch->config->get( TwatchConfig::DEFAULT_LANG ) );
				$url->setParam( 'website', $this->websiteId, 1 );
				$url->setParam( 'profile', $twatch->profile, 'default' );
				
				
				$url->setAddress( $twatch->url( $this->toRoot, '' ) );
				$this->top_buttons[ 0 ] = array( $twatch->locale->text( 'Stats' ), $url->getUrl() );
				
				$url->setAddress( $twatch->url( $this->toRoot, 'latest.php' ) );
				$this->top_buttons[ 1 ] = array( $twatch->locale->text( 'Latest Visitors' ), $url->getUrl() );
				
				$pathAnalyzerInstalled = $twatch->state->get( TwatchState::PATH_ANALYZER_INSTALLED );
				if( $pathAnalyzerInstalled ) {
					$url->setAddress( $twatch->url( $this->toRoot, 'path_analysis.php' ) );
					$this->top_buttons[ 2 ] = array( $twatch->locale->text( 'Path Analysis' ), $url->getUrl() );
				}
				if( $ardeUser->user->hasPermission( TwatchUserData::VIEW_ADMIN ) ) {
					$url->setAddress( $twatch->url( $this->toRoot, 'admin/' ) );
					$this->top_buttons[ 3 ] = array( $twatch->locale->text( 'Administrate' ), $url->getUrl() );
				}
				$url->setAddress( $twatch->url( $this->toRoot, 'about.php' ) );
				$this->top_buttons[ 4 ] = array( $twatch->locale->text( 'About' ), $url->getUrl() );
				
				

				
				ksort( $this->top_buttons );
			} else {
				$this->top_buttons = array();
			}
			
			
			parent::printHeader( $p, $twatch->locale->rightToLeft );

			if( $ardeUser->user->hasPermission( TwatchUserData::VIEW_REPORTS ) ) {
				$p->pn( '<div class="infobar"'.($this->doublePadBody?' style="margin-right:-15px"':'').'><b>'.$twatch->locale->text( 'Now' ).':</b> ' );
				$tzName = $twatch->config->get( TwatchConfig::TIME_ZONE_NAME );
				
				$dt = array();
				$dt[ 'year' ] = $twatch->now->getYear();
				$dt[ 'month' ] = $twatch->locale->text( $twatch->now->getMonthLong() );
				$dt[ 'day' ] = $twatch->now->getDay();
				$dt[ 'hour' ] = $twatch->now->getPaddedHour();
				$dt[ 'minute' ] = $twatch->now->getPaddedMinute();
				$dt[ 'second' ] = $twatch->now->getPaddedSecond();
				$dt[ 'tzname' ] = $twatch->locale->text( $tzName );
				
				$p->pn( '<span class="b_txt">'.$twatch->locale->number( $twatch->locale->text( '{year} {month} {day} | {hour}:{minute}:{second} | {tzname}', $dt ) ).'</span> <b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'.$twatch->locale->text( 'Logger Status' ).':</b> ' );
				$p->pn( '<span class="b_txt">'.$twatch->locale->text( 'Working' ) );
				
				if( $ardeUser->user->hasPermission( TwatchUserData::ADMINISTRATE ) ) {
					$p->pn( ' ( ' );
					$errorReporter = new TwatchErrorLogger();
					$errorsCount = $errorReporter->getErrorsCount();
					if( $errorsCount == 0 ) {
						$p->pn( '<span class="good light">'.$twatch->locale->text( 'No Errors' ).'</span>' );
					} else {
						$p->pn( '<a href="'.$twatch->url( $this->toRoot, 'admin/errors.php' ).'"><span class="critical">'.$errorsCount.' Error'.( $errorsCount == 1 ? '' : 's' ).'</span></a>' );
					}
					$p->pn( ' )' );
				}
				$p->pn( '</span>' );
				$p->pn( '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>'.$twatch->locale->text( 'User' ).':</b> '.$ardeUser->user->username );
				
				if( $ardeUser->user->id != ArdeUser::USER_PUBLIC ) {
					$url = new ArdeUrlWriter( 'logout.php' );
					$url->setParam( 'profile', $twatch->settings[ 'user_profile' ], 'default' );
					$p->pn( ' (<a href="'.$twatch->extUrl( $this->toRoot, 'user', $url->getUrl() ).'">'.$twatch->locale->text( 'Logout' ).'</a>)' );
				} else {
					$url = new ArdeUrlWriter( 'login.php' );
					$url->setParam( 'back', ardeRequestUri() )->setParam( 'profile', $twatch->settings[ 'user_profile' ], 'default' );
					$p->pn( ' (<a href="'.$twatch->extUrl( $this->toRoot, 'user', $url->getUrl() ).'">'.$twatch->locale->text( 'Login' ).'</a>)' );
				}
				
				if( $ardeUser->user->hasPermission( TwatchUserData::ADMINISTRATE ) ) {
					if( isset( $_COOKIE[ $twatch->settings[ 'cookie_prefix' ].'_admin' ] ) &&
						$twatch->config->get( TwatchConfig::ADMIN_COOKIE )->isAdminCookie( $_COOKIE[ $twatch->settings[ 'cookie_prefix' ].'_admin' ] ) ) {
						$p->pn( ' <span style="font-size:.8em"><span style="font-weight:normal" class="good">'.$twatch->locale->text( 'Has Admin Cookie' ).'</span></span>' );
					} else {
						$url = new ArdeUrlWriter( 'admin/admin_cookie.php' );
						$url->setParam( 'action', 'set' )->setParam( 'back', ardeRequestUri() )->setParam( 'profile', $twatch->settings[ 'user_profile' ], 'default' );
						$p->pn( ' <span style="font-size:.8em">(<a title="'.$twatch->locale->text( 'Click to set the admin cookie in your browser' ).'" href="'.$twatch->url( $this->toRoot, $url->getUrl() ).'">'.$twatch->locale->text( 'Has No Admin Cookie' ).'</a>)</span>' );
					}
				}

				
				$p->pl( '</div>' );
			}
				
		}
		
		public function printFooter( ArdePrinter $p ) {
			global $twatch;
			parent::printFooter( $p );
			if( $twatch->locale->translatorName != '' ) {
				if( $twatch->locale->translatorLink != '' ) {
					$name = '<a href="'.$twatch->locale->translatorLink.'">'.$twatch->locale->translatorName.'</a>';
				} else {
					$name = $twatch->locale->translatorName;
				}
				$p->pl( '	<div lang="en-US" xml:lang="en-US" id="translator">Translated to '.$twatch->locale->name.' by '.$name.'</div>' );
			}
				
		}
	}

	$twatch->applyOverrides( array( 'TwatchPage' => true ) );
	
?>